@@ -1,4 +1,4 @@
-// Copyright 2010 Google Inc.
+// Copyright 2010 Google Inc. All Rights Reserved.
 //
 // This code is licensed under the same terms as WebM:
 //  Software License Agreement:  http://www.webmproject.org/license/software/
@@ -38,22 +38,28 @@
 #endif
 
 #include "webp/decode.h"
-#include "stopwatch.h"
+#include "./example_util.h"
+#include "./stopwatch.h"
 
+static int verbose = 0;
+#ifndef WEBP_DLL
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
-static int verbose = 0;
-#ifndef WEBP_DLL
 extern void* VP8GetCPUInfo;   // opaque forward declaration.
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}    // extern "C"
 #endif
+#endif  // WEBP_DLL
 
 //------------------------------------------------------------------------------
 
 // Output types
 typedef enum {
   PNG = 0,
+  PAM,
   PPM,
   PGM,
   ALPHA_PLANE_ONLY  // this is for experimenting only
@@ -67,7 +73,7 @@ typedef enum {
      {                         \
         hr = (fn);             \
         if (FAILED(hr) && verbose)           \
-          printf(#fn " failed %08x\n", hr);  \
+          fprintf(stderr, #fn " failed %08x\n", hr);  \
      }                         \
   } while (0)
 
@@ -82,7 +88,7 @@ static HRESULT CreateOutputStream(const char* out_file_name,
   HRESULT hr = S_OK;
   IFS(SHCreateStreamOnFileA(out_file_name, STGM_WRITE | STGM_CREATE, ppStream));
   if (FAILED(hr))
-    printf("Error opening output file %s (%08x)\n", out_file_name, hr);
+    fprintf(stderr, "Error opening output file %s (%08x)\n", out_file_name, hr);
   return hr;
 }
 
@@ -102,9 +108,10 @@ static HRESULT WriteUsingWIC(const char* out_file_name, REFGUID container_guid,
           CLSCTX_INPROC_SERVER, MAKE_REFGUID(IID_IWICImagingFactory),
           (LPVOID*)&pFactory));
   if (hr == REGDB_E_CLASSNOTREG) {
-    printf("Couldn't access Windows Imaging Component (are you running \n");
-    printf("Windows XP SP3 or newer?). PNG support not available.\n");
-    printf("Use -ppm or -pgm for available PPM and PGM formats.\n");
+    fprintf(stderr,
+            "Couldn't access Windows Imaging Component (are you running "
+            "Windows XP SP3 or newer?). PNG support not available. "
+            "Use -ppm or -pgm for available PPM and PGM formats.\n");
   }
   IFS(CreateOutputStream(out_file_name, &pStream));
   IFS(IWICImagingFactory_CreateEncoder(pFactory, container_guid, NULL,
@@ -185,28 +192,32 @@ static int WritePNG(FILE* out_file, const WebPDecBuffer* const buffer) {
   return 1;
 }
 #else    // !HAVE_WINCODEC_H && !WEBP_HAVE_PNG
-
-typedef uint32_t png_uint_32;
-
 static int WritePNG(FILE* out_file, const WebPDecBuffer* const buffer) {
   (void)out_file;
   (void)buffer;
-  printf("PNG support not compiled. Please install the libpng development "
-         "package before building.\n");
-  printf("You can run with -ppm flag to decode in PPM format.\n");
+  fprintf(stderr, "PNG support not compiled. Please install the libpng "
+          "development package before building.\n");
+  fprintf(stderr, "You can run with -ppm flag to decode in PPM format.\n");
   return 0;
 }
 #endif
 
-static int WritePPM(FILE* fout, const WebPDecBuffer* const buffer) {
+static int WritePPM(FILE* fout, const WebPDecBuffer* const buffer, int alpha) {
   const uint32_t width = buffer->width;
   const uint32_t height = buffer->height;
   const unsigned char* const rgb = buffer->u.RGBA.rgba;
   const int stride = buffer->u.RGBA.stride;
+  const size_t bytes_per_px = alpha ? 4 : 3;
   uint32_t y;
-  fprintf(fout, "P6\n%d %d\n255\n", width, height);
+
+  if (alpha) {
+    fprintf(fout, "P7\nWIDTH %d\nHEIGHT %d\nDEPTH 4\nMAXVAL 255\n"
+                  "TUPLTYPE RGB_ALPHA\nENDHDR\n", width, height);
+  } else {
+    fprintf(fout, "P6\n%d %d\n255\n", width, height);
+  }
   for (y = 0; y < height; ++y) {
-    if (fwrite(rgb + y * stride, width, 3, fout) != 3) {
+    if (fwrite(rgb + y * stride, width, bytes_per_px, fout) != bytes_per_px) {
       return 0;
     }
   }
@@ -286,8 +297,10 @@ static void SaveOutput(const WebPDecBuffer* const buffer,
 #else
     ok &= WritePNG(fout, buffer);
 #endif
+  } else if (format == PAM) {
+    ok &= WritePPM(fout, buffer, 1);
   } else if (format == PPM) {
-    ok &= WritePPM(fout, buffer);
+    ok &= WritePPM(fout, buffer, 0);
   } else if (format == PGM) {
     ok &= WritePGM(fout, buffer);
   } else if (format == ALPHA_PLANE_ONLY) {
@@ -311,7 +324,8 @@ static void Help(void) {
   printf("Usage: dwebp in_file [options] [-o out_file]\n\n"
          "Decodes the WebP image file to PNG format [Default]\n"
          "Use following options to convert into alternate image formats:\n"
-         "  -ppm ......... save the raw RGB samples as color PPM\n"
+         "  -pam ......... save the raw RGBA samples as a color PAM\n"
+         "  -ppm ......... save the raw RGB samples as a color PPM\n"
          "  -pgm ......... save the raw YUV samples as a grayscale PGM\n"
          "                 file with IMC4 layout.\n"
          " Other options are:\n"
@@ -321,9 +335,7 @@ static void Help(void) {
          "  -mt .......... use multi-threading\n"
          "  -crop <x> <y> <w> <h> ... crop output with the given rectangle\n"
          "  -scale <w> <h> .......... scale the output (*after* any cropping)\n"
-#ifdef WEBP_EXPERIMENTAL_FEATURES
          "  -alpha ....... only save the alpha plane.\n"
-#endif
          "  -h     ....... this help message.\n"
          "  -v     ....... verbose (e.g. print encoding/decoding times)\n"
 #ifndef WEBP_DLL
@@ -364,12 +376,14 @@ int main(int argc, const char *argv[]) {
       config.options.no_fancy_upsampling = 1;
     } else if (!strcmp(argv[c], "-nofilter")) {
       config.options.bypass_filtering = 1;
+    } else if (!strcmp(argv[c], "-pam")) {
+      format = PAM;
     } else if (!strcmp(argv[c], "-ppm")) {
       format = PPM;
     } else if (!strcmp(argv[c], "-version")) {
       const int version = WebPGetDecoderVersion();
       printf("%d.%d.%d\n",
-        (version >> 16) & 0xff, (version >> 8) & 0xff, version & 0xff);
+             (version >> 16) & 0xff, (version >> 8) & 0xff, version & 0xff);
       return 0;
     } else if (!strcmp(argv[c], "-pgm")) {
       format = PGM;
@@ -392,7 +406,7 @@ int main(int argc, const char *argv[]) {
       VP8GetCPUInfo = NULL;
 #endif
     } else if (argv[c][0] == '-') {
-      printf("Unknown option '%s'\n", argv[c]);
+      fprintf(stderr, "Unknown option '%s'\n", argv[c]);
       Help();
       return -1;
     } else {
@@ -401,7 +415,7 @@ int main(int argc, const char *argv[]) {
   }
 
   if (in_file == NULL) {
-    printf("missing input file!!\n");
+    fprintf(stderr, "missing input file!!\n");
     Help();
     return -1;
   }
@@ -410,31 +424,15 @@ int main(int argc, const char *argv[]) {
     Stopwatch stop_watch;
     VP8StatusCode status = VP8_STATUS_OK;
     int ok;
-    uint32_t data_size = 0;
-    void* data = NULL;
-    FILE* const in = fopen(in_file, "rb");
+    size_t data_size = 0;
+    const uint8_t* data = NULL;
 
-    if (!in) {
-      fprintf(stderr, "cannot open input file '%s'\n", in_file);
-      return 1;
-    }
-    fseek(in, 0, SEEK_END);
-    data_size = ftell(in);
-    fseek(in, 0, SEEK_SET);
-    data = malloc(data_size);
-    ok = (fread(data, data_size, 1, in) == 1);
-    fclose(in);
-    if (!ok) {
-      fprintf(stderr, "Could not read %d bytes of data from file %s\n",
-              data_size, in_file);
-      free(data);
-      return -1;
-    }
+    if (!ExUtilReadFile(in_file, &data, &data_size)) return -1;
 
     if (verbose)
       StopwatchReadAndReset(&stop_watch);
 
-    status = WebPGetFeatures((const uint8_t*)data, data_size, bitstream);
+    status = WebPGetFeatures(data, data_size, bitstream);
     if (status != VP8_STATUS_OK) {
       goto end;
     }
@@ -447,6 +445,9 @@ int main(int argc, const char *argv[]) {
         output_buffer->colorspace = bitstream->has_alpha ? MODE_RGBA : MODE_RGB;
 #endif
         break;
+      case PAM:
+        output_buffer->colorspace = MODE_RGBA;
+        break;
       case PPM:
         output_buffer->colorspace = MODE_RGB;  // drops alpha for PPM
         break;
@@ -457,17 +458,17 @@ int main(int argc, const char *argv[]) {
         output_buffer->colorspace = MODE_YUVA;
         break;
       default:
-        free(data);
+        free((void*)data);
         return -1;
     }
-    status = WebPDecode((const uint8_t*)data, data_size, &config);
+    status = WebPDecode(data, data_size, &config);
 
     if (verbose) {
       const double time = StopwatchReadAndReset(&stop_watch);
       printf("Time to decode picture: %.3fs\n", time);
     }
  end:
-    free(data);
+    free((void*)data);
     ok = (status == VP8_STATUS_OK);
     if (!ok) {
       fprintf(stderr, "Decoding of %s failed.\n", in_file);
@@ -493,7 +494,3 @@ int main(int argc, const char *argv[]) {
 }
 
 //------------------------------------------------------------------------------
-
-#if defined(__cplusplus) || defined(c_plusplus)
-}    // extern "C"
-#endif
